#~ '(C) Links System Software 2009
#~ 'Feedback information
#~ 'www: http://lss2008.livejournal.com/
#~ 'E-mail1: designer@ls-software.ru
#~ 'E-mail2: kirill2007_77@mail.ru
#~ 'icq: 328-958-369

#~ pathface.rb ver. 1.3. 01-Oct-09
#~ Plug-in, which creates blended object from 2 faces + path curve

#~ How to use:
#~ 1)	put pathface.rb to Google SketchUp 6\Plugins folder
#~ 2)	start SketchUp 6
#~ 3)	draw 2 faces and one curve then select them
#~ 4)	choose Plugins>> 2 Faces + Path command from the main menu or "2 Faces + Path" command from the context menu

require 'sketchup.rb'



class Path_Face_ProgressBar 

@@err_total_notnumeric = "ProgressBar: Total must be a positive integer."
@@err_count_notnumeric = "ProgressBar: Iteration Count must be numeric"
@@end_time             = "End @ " 
@@progresschar = ">" ; 
@@initial_block = "-" * 50     # Default progress bar line sequence.

def initialize(total,phase=nil)  
  if (!total.integer? or total < 0)
    raise(ArgumentError,@@err_total_notnumeric) 
    return ; 
    end ; 
  @total = total.to_i ; 
  @phase = phase ; 
  @firsttime = true ; 
  end ; 

def update(iteration) 
  if !iteration.integer? 
    raise(ArgumentError,@@err_count_notnumeric)  
    return ; 
    end ; 
  iteration = [iteration.abs,@total].min  # make sure we don't exceed the total count or have a value < 0
  pct = [1,(iteration*100)/@total].max    # Calculate percentage complete.
                                          # round up to 1% if anything less than 1%.
  end_time = "?" ; 
  if @firsttime then ; 
    # Get the current time of day. 
    # set up an elapsed timer so we can calculate expected end time.  
    @time1 = Time.now ;     # Get current time of day. 
    @firsttime = false ;    # turn off switch 
  else 
    # divide the elapsed time by the pct complete, then multiple that by 100, then add that to the 
    # start time, and that is the expected end time.  
    end_time = Time.at(((((Time.now-@time1).to_f)/pct)*100 + @time1.to_f).to_i).strftime("%H:%M:%S") 
    end ; 
  pct_pos = [pct/2,1].max ;
  current_block = @@initial_block[0,pct_pos-1] << @@progresschar << @@initial_block[pct_pos,@@initial_block.length]
  Sketchup.set_status_text(current_block << "   " << (pct.to_s) << "%. #{@@end_time} " << end_time << " #{@phase}")
  Sketchup.set_status_text(current_block << "   " << (pct.to_s) << "%")
  # if (((Time.now-@time1).to_f/pct)*100) > $cbt_delay
    # $cbt_stop_timer = true
  # end
  end ; 

end ; # class ProgressBar ;

class PathFaceEntityObserver < Sketchup::EntityObserver 
    def initialize (ent1,ent2, path, draw_surf_flag, draw_tracks_flag, blend_skin_color, align_flag,enforce_flag,contours_group,mesh_group,tracks_group,adj_curve,adj_times)
      @ent1=ent1
      @ent2=ent2
      @path=path
      @draw_tracks_flag=draw_tracks_flag
      @blend_skin_color=blend_skin_color
      @draw_surf_flag=draw_surf_flag
      @align_flag=align_flag
      @enforce_flag=enforce_flag
      @contours_group=contours_group
      @mesh_group=mesh_group
      @tracks_group=tracks_group
      @adj_curve=adj_curve
      @adj_times=adj_times
      @redraw_pathface=Lss_Path_Face.new
      @on_change_cnt=0
    end
   def onChangeEntity(entity)
      @on_change_cnt+=1
      puts "The Initial #{entity.typename} Change No: #{@on_change_cnt}"
     begin
        if not @ent1.nil? and not @ent2.nil? and not @ent1.deleted? and not @ent2.deleted?
          
          @contours_group.entities.clear! if not @contours_group.nil? and not @contours_group.deleted?
          @mesh_group.entities.clear! if not @mesh_group.nil? and not @mesh_group.deleted?
          @tracks_group.entities.clear! if not @tracks_group.nil? and not @tracks_group.deleted?
   
          @redraw_pathface.generate @draw_surf_flag, @draw_tracks_flag, @blend_skin_color,@align_flag,@enforce_flag,@ent1, @ent2, @path, @contours_group,@mesh_group,@tracks_group,@adj_curve,@adj_times
        end
      rescue
        puts "onChangeEntity handle aborted"
        Sketchup.active_model.abort_operation
      end
    end 
  end 

class Lss_Path_Face
    def initialize
      @cont_grp_faces_cnt=0
      @bad_int_face_cnt=0
      @curve_instead_cnt=0
      @mesh_grp_faces_cnt=0
      @tracks_grp_edges_cnt=0
      @bad_face_cnt=0
      @path_verts_cnt=0
    end
    #mega selection tool neglects none (almost)
    def selection_filter
      @model=Sketchup.active_model
      @selection=@model.selection
      #searching for 2 faces
      #first face
      ind=0
      while ind<@selection.count
        @selected_type=@selection[ind].typename
        face1=@selection[ind] if @selected_type =="Face"
        ind +=1
      end

      #second face
      ind=0
      while ind<@selection.count 
        @selected_type=@selection[ind].typename
        face2=@selection[ind] if @selected_type =="Face" and not @selection[ind]==face1
        ind +=1
      end

      #path curve
      ind=0
      while @path.nil? and ind<@selection.count
        if @selection[ind]#.respond_to? "all_connected"
          @path=@selection[ind].curve if @selection[ind].respond_to? "curve"
          if @path
            coincide_cnt1=0
            coincide_cnt2=0
            @path.vertices.each{|vert|
              result1 = face1.classify_point(vert.position)
              result2 = face2.classify_point(vert.position)
              coincide_cnt1+=1 if result1==4
              coincide_cnt2+=1 if result2==4
            }
            erase_path=false
            if coincide_cnt1==@path.vertices.length or coincide_cnt2==@path.vertices.length
              erase_path=true 
              #~ puts "erase"
            end
            @path=nil if erase_path
          end
        end
        ind +=1
      end
      
      ###################
      #   finding start and end face   #
      ###################
      if face1 and face2
        face1_dist=0
        face2_dist=0
        face1.vertices.each{|vert|
          face1_dist+=vert.position.distance @path.vertices[0].position
        }      
        face2.vertices.each{|vert|
          face2_dist+=vert.position.distance @path.vertices[0].position
        }      
        if face1_dist<face2_dist
          @face1=face1
          @face2=face2
        else
          @face1=face2
          @face2=face1
        end
        ##################
      end
      
      if @face1.nil? or @face2.nil? or @path.nil?
        UI.messagebox "It is necessary to select 2 faces and one curve"
        return
      end
    end
    
    def align_to_xy(curve)
      entities=Sketchup.active_model.entities
      vert_ind=1
      norm=Geom::Vector3d.new 
      while vert_ind<curve.vertices.length-1
        vec1=curve.vertices[0].position.vector_to curve.vertices[vert_ind].position
        vec2=curve.vertices[0].position.vector_to curve.vertices[vert_ind+1].position
        vert_ind+=1
        norm+=vec1.cross vec2
        norm.normalize!
      end
      @curve_transform=Geom::Transformation.new [0,0,0],norm if norm.length>0
      curve_align_xy=@curve_transform.inverse if @curve_transform
      entities.transform_entities(curve_align_xy, curve) if @curve_transform
    end
    
    #######################################################
    #                                                                                                                                     #
    #                                                  main 'generation' method                                                # 
    #                                                                                                                                    #
    #######################################################
    def generate(draw_surf_flag, draw_tracks_flag, blend_skin_color, align_flag="Yes", enforce_flag="...", fc1=nil, fc2=nil, pth=nil, contours_group=nil,mesh_group=nil,tracks_group=nil,adj_curve=nil,adj_times=1)
      
      @cont_grp_faces_cnt=0
      @bad_int_face_cnt=0
      @curve_instead_cnt=0
      @mesh_grp_faces_cnt=0
      @tracks_grp_edges_cnt=0
      @bad_face_cnt=0
      @path_verts_cnt=0
      
      if fc1==nil and fc2==nil
        make_observer_flag=true 
      else
        make_observer_flag=false
      end
      
      tracks_group = Sketchup.active_model.entities.add_group if tracks_group==nil or tracks_group.deleted?
      mesh_group = Sketchup.active_model.entities.add_group if mesh_group==nil or mesh_group.deleted?
      contours_group = Sketchup.active_model.entities.add_group if contours_group==nil or contours_group.deleted?
      
      tracks_group.description="lss pathface tracks" if not tracks_group.deleted?
      mesh_group.description="lss pathface mesh" if not mesh_group.deleted?
      contours_group.description="lss pathface contours" if not contours_group.deleted?
      
      tracks_group.transform! tracks_group.transformation.inverse
      mesh_group.transform! mesh_group.transformation.inverse
      contours_group.transform! contours_group.transformation.inverse
      
      entities = Sketchup.active_model.entities
      
      points1=[]
      points2=[]
      points1=@face1.vertices if @face1
      points2=@face2.vertices if @face2

      if not fc1.nil? and not fc2.nil? #and points1.nil? and points2.nil?
        points1=fc1.vertices if fc1.typename=="Face"
        points2=fc2.vertices if fc2.typename=="Face"
      end
      face1=@face1
      face2=@face2
      
      if not fc1.nil? and not fc2.nil? #and face1.nil? and face2.nil?
        face1=fc1 if fc1.typename=="Face"
        face2=fc2 if fc2.typename=="Face"
      end
      
      if pth.nil?
        path=@path
      else
        path=pth
      end

      if not face1.nil? and not face2.nil?
        mat1=face1.material
        mat2=face2.material
        bk_mat1=face1.back_material
        bk_mat2=face2.back_material
        ######################
        col1=mat1.color if mat1
        col2=mat2.color if mat2
        bk_col1=bk_mat1.color if bk_mat1
        bk_col2=bk_mat2.color if bk_mat2
        ######################
      end
      if points1.length>=points2.length
        max_pnts_cnt=points1.length
      else
        max_pnts_cnt=points2.length
      end

      step=0
      verts1=nil
      
      status = Sketchup.active_model.start_operation "2 faces + path"
      
      align_to_xy adj_curve if not adj_curve.nil?
      
      steps=path.edges.length
      
      path_start=[face1.bounds.center.x-path.first_edge.start.position.x,face1.bounds.center.y-path.first_edge.start.position.y,face1.bounds.center.z-path.first_edge.start.position.z]
      path_end=[face2.bounds.center.x-path.last_edge.end.position.x,face2.bounds.center.y-path.last_edge.end.position.y,face2.bounds.center.z-path.last_edge.end.position.z]
      ########################################
      #            finding path plane to find out if the path curve is flat        #
      ########################################
      path_is_flat=true
      vert_ind=0
      path_pnts=[]
      path.vertices.each{|vrt| path_pnts+=[vrt.position]}
      path_line=nil
      begin
        path_line = [path.vertices[0].position, path.vertices[-1].position]
      rescue
        puts "Path curve is closed"
      end
      path_is_line=true
      path_is_closed=false
      if path_line
        if path_line[0]==path_line[1]
          puts "Path curve is closed"
          path_is_closed=true
          path_is_line=false
        else
          for vrt in path.vertices
            if not vrt.position.on_line? path_line
              path_is_line=false
              break
            end
          end
        end
      end
      path_plane=nil
      begin
        path_plane = Geom.fit_plane_to_points path_pnts
      rescue
        puts "Path curve has no plane (most likely, all vertices are at the same line)"
      end
      if path_plane
        for vrt in path.vertices
          if not vrt.position.on_plane? path_plane
            path_is_flat=false
            break
          end
        end
      end
      if path_is_line
        puts "Path curve is straight line"
        has_flat_parts=false
      else
        if path_is_flat
          puts "Path curve is flat" 
        else
          puts "Path curve is not flat (3D path)" 
        end
      end
      if path_is_flat and not path_is_line
        vert_ind=1
        norm=Geom::Vector3d.new 
        while vert_ind<path.vertices.length-1
          vec1=path.vertices[0].position.vector_to path.vertices[vert_ind].position
          vec2=path.vertices[0].position.vector_to path.vertices[vert_ind+1].position
          vert_ind+=1
          norm+=vec1.cross vec2
          norm.normalize!
        end
        path_transform=Geom::Transformation.new [0,0,0],norm if norm.length>0
        path_align_xy=path_transform.inverse
      end
      ###################################
      # analize non flat path curve to search for flat parts in it
      ###################################
      path_flat_parts=[]
      flat_parts_norms=[]
      ind_prev=0
      flat_part_ind=0
      path_plane=nil
      if not path_is_flat and not path_is_line
        vert_ind=0
        while vert_ind<path.vertices.length-1
          path_pnts+=[path.vertices[vert_ind].position] if path.vertices[vert_ind]
          path_plane = Geom.fit_plane_to_points path_pnts if path_pnts.length>2
          flat_curve_part=nil
          flat_curve_part=[]
          if path_plane and path.vertices[vert_ind+1]
            if not path.vertices[vert_ind+1].position.on_plane? path_plane or vert_ind==path.vertices.length-2
              new_ind=ind_prev
              while new_ind<vert_ind
                flat_curve_part+=[path.vertices[new_ind].position]
                new_ind+=1
              end
              path_flat_parts+=[flat_curve_part]
              fl_prt_nrm=Geom::Vector3d.new 
              fl_vert_ind=0
              while fl_vert_ind<flat_curve_part.length-1
                vec1=flat_curve_part[0].vector_to flat_curve_part[fl_vert_ind]
                vec2=flat_curve_part[0].vector_to flat_curve_part[fl_vert_ind+1]
                fl_vert_ind+=1
                fl_prt_nrm+=vec1.cross vec2
                fl_prt_nrm.normalize!
              end
              flat_parts_norms+=[fl_prt_nrm]
              #it is necessary to avoid some problems with flat parts
              if flat_part_ind>0
                flat_parts_norms[flat_part_ind].reverse! if flat_parts_norms[flat_part_ind].angle_between(flat_parts_norms[flat_part_ind-1])>1.5707963267949
              end
              ###############################
              flat_part_ind+=1
              path_pnts=nil
              path_pnts=[]
              path_plane=nil
              ind_prev=vert_ind+1
              vert_ind+=1
            end
          end
          vert_ind+=1
        end
        flat_parts_norms.delete_at(0) #it is necessary to delete the first element (it is empty anyway)
        path_flat_parts.delete_at(0) #it is necessary to delete the first element (it is empty anyway)
        #checking if the flat parts count is not the same as vertices count divided by 4
        if not path_flat_parts.length.to_i==((path.vertices.length+1)/4-1).to_i 
          puts "Flat parts count: \t \t \t #{path_flat_parts.length} \n"
          has_flat_parts=true
        else
          puts "Look's like the path curve has no flat parts (more than 3 coplanar points pieces)"
          has_flat_parts=false
        end  
      end

      @pb=Path_Face_ProgressBar.new(steps)
      ########################
      z_vec_prev=Geom::Vector3d.new
      x_vec_prev=Geom::Vector3d.new
      z_vec_len=true
      reverse_flag=false
      ########################
      flip_flag=false
      move3_prev=Geom::Transformation.new
      move3_tr_prev=Geom::Transformation.new
      flip=0
      chk_ang_prev=0
      if not enforce_flag=="..."
        if enforce_flag=="Enforce 3d algorithm"
          has_flat_parts=false
          path_is_flat=false
          #~ path_is_line=false
        else
          has_flat_parts=true
        end
      end
      if align_flag=="Yes" or align_flag=="Yes and center to path" and path_is_flat and not path_is_line and not enforce_flag=="Enforce 3d algorithm"
        entities.transform_entities(path_align_xy, path)
        path_start.transform! path_align_xy
        path_end.transform! path_align_xy
      end
      path_pnts=Array.new
      path.vertices.each{|vrt| path_pnts+=[vrt.position]}
      path_pnts.uniq!
      path_pnts.compact!
      
      face_bound1=face1.bounds
      face_bound2=face2.bounds
      
      init_points_1=face1.vertices
      init_points_2=face2.vertices
      
      face1_tr=Geom::Transformation.new [0,0,0],face1.normal
      face2_tr=Geom::Transformation.new [0,0,0],face2.normal
      
      points_copy1=[]; points_copy2=[]

      if align_flag=="Yes" or align_flag=="Yes and center to path"
        init_points_1.each{|vrt| points_copy1+=[vrt.position.transform(face1_tr.inverse)]}
        init_points_2.each{|vrt| points_copy2+=[vrt.position.transform(face2_tr.inverse)]}
      else
        init_points_1.each{|vrt| points_copy1+=[vrt.position]}
        init_points_2.each{|vrt| points_copy2+=[vrt.position]}
      end
      curr_fl_prt_idx=0
      fl_prt_start_step=0
      nrm_flip_flag=false
      chk_ang_prev_z=0
      #################################################################
      #                                       main while loop: processing path curve vertices                                                      #
      #################################################################
      while step<=steps
        if has_flat_parts and (align_flag=="Yes" or align_flag=="Yes and center to path") and not enforce_flag=="Ignore flat parts orientation"
          if path_flat_parts[curr_fl_prt_idx]
            if step>path_flat_parts[curr_fl_prt_idx].length+1+fl_prt_start_step
              curr_fl_prt_idx+=1
              fl_prt_start_step=step-1
            end
          end
          curr_nrm_init=flat_parts_norms[curr_fl_prt_idx]
          if curr_nrm_init
            if curr_fl_prt_idx<path_flat_parts.length and step<steps-2 and curr_fl_prt_idx>0
              #~ chk_step=fl_prt_start_step+2
              chk_step=step+1
              x11=path_pnts[chk_step-2].x-path_pnts[chk_step-1].x 
              y11=path_pnts[chk_step-2].y-path_pnts[chk_step-1].y 
              z11=path_pnts[chk_step-2].z-path_pnts[chk_step-1].z
              x22=path_pnts[chk_step-1].x-path_pnts[chk_step].x 
              y22=path_pnts[chk_step-1].y-path_pnts[chk_step].y 
              z22=path_pnts[chk_step-1].z-path_pnts[chk_step].z
              x33=path_pnts[chk_step].x-path_pnts[chk_step+1].x 
              y33=path_pnts[chk_step].y-path_pnts[chk_step+1].y 
              z33=path_pnts[chk_step].z-path_pnts[chk_step+1].z
              x44=path_pnts[chk_step].x-path_pnts[chk_step+1].x 
              y44=path_pnts[chk_step].y-path_pnts[chk_step+1].y 
              z44=path_pnts[chk_step].z-path_pnts[chk_step+1].z
              segm11=Geom::Vector3d.new x11,y11,z11
              segm22=Geom::Vector3d.new x22,y22,z22
              segm33=Geom::Vector3d.new x33,y33,z33
              segm44=Geom::Vector3d.new x44,y44,z44
              segm11.normalize!;segm22.normalize!; segm33.normalize!;segm44.normalize!
              z_vec_temp1=Geom::Vector3d.new segm11.x+segm22.x,segm11.y+segm22.y,segm11.z+segm22.z
              z_vec_temp2=Geom::Vector3d.new segm22.x+segm33.x,segm22.y+segm33.y,segm22.z+segm33.z
              z_vec_temp3=Geom::Vector3d.new segm33.x+segm44.x,segm33.y+segm44.y,segm33.z+segm44.z
              x_vec_chk1=z_vec_temp1.cross flat_parts_norms[curr_fl_prt_idx-1]
              x_vec_chk2=z_vec_temp3.cross curr_nrm_init
              z_vec_chk=x_vec_chk1.cross x_vec_chk2
              chk_ang_z=z_vec_chk.angle_between z_vec_temp3
              if (chk_ang_z-Math::PI).abs<0.0001 and step==fl_prt_start_step+1
                if nrm_flip_flag==false
                  nrm_flip_flag=true
                else
                  nrm_flip_flag=false
                end
              end
              chk_ang_prev_z=z_vec_chk.angle_between z_vec_temp3
            end
            
            if nrm_flip_flag
              curr_nrm=Geom::Vector3d.new(curr_nrm_init.reverse)
            else
              curr_nrm=Geom::Vector3d.new(curr_nrm_init) 
            end
            curr_fl_prt_trnsfrm=Geom::Transformation.new [0,0,0],curr_nrm if curr_nrm.length>0
            curr_fl_prt_align_xy=curr_fl_prt_trnsfrm.inverse
          end
        end
        if align_flag=="Yes" or align_flag=="Yes and center to path"
          points_copy1=[]; points_copy2=[]
          init_points_1.each{|vrt| points_copy1+=[vrt.position]}
          init_points_2.each{|vrt| points_copy2+=[vrt.position]}
        end
        
        pnt_ind=0
        verts=[]

        x1=0; y1=0; z1=0
        x1=path_pnts[step-1].x-path_pnts[step].x if step>0 
        y1=path_pnts[step-1].y-path_pnts[step].y if step>0 
        z1=path_pnts[step-1].z-path_pnts[step].z if step>0 
        x2=0; y2=0; z2=0
        x2=path_pnts[step].x-path_pnts[step+1].x if step<steps
        y2=path_pnts[step].y-path_pnts[step+1].y if step<steps
        z2=path_pnts[step].z-path_pnts[step+1].z if step<steps
        
        if path_is_closed
          if step==0
            x1=path_pnts[steps-1].x-path_pnts[steps].x
            y1=path_pnts[steps-1].y-path_pnts[steps].y 
            z1=path_pnts[steps-1].z-path_pnts[steps].z
            
            x2=path_pnts[0].x-path_pnts[1].x
            y2=path_pnts[0].y-path_pnts[1].y
            z2=path_pnts[0].z-path_pnts[1].z
          end
          if step==steps
            x1=path_pnts[step-1].x-path_pnts[step].x
            y1=path_pnts[step-1].y-path_pnts[step].y 
            z1=path_pnts[step-1].z-path_pnts[step].z
            
            x2=path_pnts[step].x-path_pnts[1].x
            y2=path_pnts[step].y-path_pnts[1].y
            z2=path_pnts[step].z-path_pnts[1].z
          end
        end

        segm1_vec=Geom::Vector3d.new x1,y1,z1
        segm2_vec=Geom::Vector3d.new x2,y2,z2
        segm1_vec.normalize!#added ver. 1.3. to aling to path regardless of segment length
        segm2_vec.normalize!#maybe it is better to make it optional for any reason?
        z_vec=Geom::Vector3d.new segm1_vec.x+segm2_vec.x,segm1_vec.y+segm2_vec.y,segm1_vec.z+segm2_vec.z
        
        #~ puts segm1_vec
        #~ puts segm2_vec
        #~ puts "step: #{step}; z_vec: #{z_vec}"
        origin_pt=path.vertices[step].position if path.vertices[step]
        if path_is_flat or has_flat_parts or path_is_line
          if has_flat_parts and align_flag=="Yes" or align_flag=="Yes and center to path" and not path_is_flat and not path_is_line
            z_vec.transform!(curr_fl_prt_align_xy) if not enforce_flag=="Ignore flat parts orientation"
          end
          move3=Geom::Transformation.new origin_pt,z_vec if not (z_vec.x==0 and z_vec.y==0 and z_vec.z==0)
          if move3
            x_axis2=move3.xaxis; y_axis2=move3.yaxis; z_axis2=move3.zaxis
            x_axis_prev=move3_prev.xaxis; y_axis_prev=move3_prev.yaxis; z_axis_prev=move3_prev.zaxis
            if z_axis2.x.abs<0.001 and z_axis2.y.abs<0.001 and not path_is_line
              if step>0
                x_axis2.x=x_axis_prev.x; x_axis2.y=x_axis_prev.y
                y_axis2.x=y_axis_prev.x; y_axis2.y=y_axis_prev.y
                if flip==1
                  x_axis2.reverse!
                  y_axis2.reverse!
                end
                move3=Geom::Transformation.new x_axis2,y_axis2,z_axis2,origin_pt
              else
                if not path.vertices==path.vertices.uniq
                  x_axis2=segm1_vec.cross segm2_vec
                  z_axis2=[0,0,-segm2_vec.z]
                  y_axis2=x_axis2.cross z_axis2
                end
                move3=Geom::Transformation.new x_axis2,y_axis2,z_axis2,origin_pt
              end
            end
            chk_ang=x_axis2.angle_between x_axis_prev.reverse
            if chk_ang==0
              flip=1
            end
            if (chk_ang-3.141592).abs<0.01
              flip=0
            end
            if flip==1
              x_axis2.reverse!
              y_axis2.reverse!
              move3=Geom::Transformation.new x_axis2,y_axis2,z_axis2,origin_pt
            end
          end
          move3_prev=Geom::Transformation.new x_axis2,y_axis2,z_axis2,origin_pt
        else
          x_vec=segm1_vec.cross segm2_vec
          #maybe it is not necessary
          if x_vec.length<0.1
            x_vec=x_vec_prev
          end
          ###############
          y_vec=z_vec.cross x_vec
          if path.vertices==path.vertices.uniq #it means that path curve is not closed
            if step==0
              x1=path_pnts[0].x-path_pnts[1].x
              y1=path_pnts[0].y-path_pnts[1].y 
              z1=path_pnts[0].z-path_pnts[1].z
              
              x2=path_pnts[1].x-path_pnts[2].x
              y2=path_pnts[1].y-path_pnts[2].y
              z2=path_pnts[1].z-path_pnts[2].z
              segm1_vec=Geom::Vector3d.new x1,y1,z1
              segm2_vec=Geom::Vector3d.new x2,y2,z2
              segm1_vec.normalize!#added ver. 1.3. to aling to path regardless of segment length
              segm2_vec.normalize!#maybe it is better to make it optional for any reason?
              z_vec=Geom::Vector3d.new segm1_vec.x+segm2_vec.x,segm1_vec.y+segm2_vec.y,segm1_vec.z+segm2_vec.z
              x_vec=segm1_vec.cross segm2_vec
              if x_vec.length<0.5 and (enforce_flag=="Enforce 3d algorithm" and path_is_flat or has_flat_parts)
                x_vec=x_vec_prev
              end
              y_vec=z_vec.cross x_vec
            end
            if step==steps
              x1=path_pnts[step-2].x-path_pnts[step-1].x
              y1=path_pnts[step-2].y-path_pnts[step-1].y 
              z1=path_pnts[step-2].z-path_pnts[step-1].z
              
              x2=path_pnts[step-1].x-path_pnts[step].x
              y2=path_pnts[step-1].y-path_pnts[step].y
              z2=path_pnts[step-1].z-path_pnts[step].z
              segm1_vec=Geom::Vector3d.new x1,y1,z1
              segm2_vec=Geom::Vector3d.new x2,y2,z2
              segm1_vec.normalize!#added ver. 1.3. to aling to path regardless of segment length
              segm2_vec.normalize!#maybe it is better to make it optional for any reason?
              z_vec=Geom::Vector3d.new segm1_vec.x+segm2_vec.x,segm1_vec.y+segm2_vec.y,segm1_vec.z+segm2_vec.z
              x_vec=segm1_vec.cross segm2_vec
              #maybe it is not necessary
              if x_vec.length<0.1
                x_vec=x_vec_prev
              end
              ###############
              y_vec=z_vec.cross x_vec
            end
          end
          begin
            move3=Geom::Transformation.new x_vec,y_vec,z_vec,origin_pt if not (z_vec.x==0 and z_vec.y==0 and z_vec.z==0)
          rescue
            move3=Geom::Transformation.new origin_pt,z_vec if not (z_vec.x==0 and z_vec.y==0 and z_vec.z==0)
          end
        end
        if align_flag=="Yes" or align_flag=="Yes and center to path"
          if has_flat_parts and not path_is_flat and not path_is_line and not enforce_flag=="Ignore flat parts orientation"
            x_ax=move3.xaxis; y_ax=move3.yaxis; z_ax=move3.zaxis
            x_ax_tr=x_ax.transform(curr_fl_prt_trnsfrm)
            y_ax_tr=y_ax.transform(curr_fl_prt_trnsfrm)
            z_ax_tr=z_ax.transform(curr_fl_prt_trnsfrm)
            move3_tr=Geom::Transformation.new x_ax_tr,y_ax_tr,z_ax_tr,origin_pt
            points_copy1.each{|pnt| pnt.transform!(move3_tr)} if move3_tr
            points_copy2.each{|pnt| pnt.transform!(move3_tr)} if move3_tr
          else
            points_copy1.each{|pnt| pnt.transform!(move3)} if move3
            points_copy2.each{|pnt| pnt.transform!(move3)} if move3
          end
        end
        new_bounds1=Geom::BoundingBox.new
        new_bounds2=Geom::BoundingBox.new
        points_copy1.each{|pnt| new_bounds1.add(pnt)} 
        points_copy2.each{|pnt| new_bounds2.add(pnt)}
        if align_flag=="Yes" or align_flag=="No"
          orgn_vert3=path.vertices[step].position-new_bounds1.center+path_start if path.vertices[step]
          orgn_vert4=path.vertices[step].position-new_bounds2.center+path_end if path.vertices[step]
        else
          orgn_vert3=path.vertices[step].position-new_bounds1.center if path.vertices[step]
          orgn_vert4=path.vertices[step].position-new_bounds2.center if path.vertices[step]
        end
        
        move1=Geom::Transformation.new orgn_vert3
        move2=Geom::Transformation.new orgn_vert4

        points_copy1.each{|pnt| pnt.transform!(move1)}
        points_copy2.each{|pnt| pnt.transform!(move2)}
      
        while pnt_ind<max_pnts_cnt
          pnt_ind1=(pnt_ind*points1.length/max_pnts_cnt).round
          pnt_ind2=(pnt_ind*points2.length/max_pnts_cnt).round
          vert1=points_copy1[pnt_ind1] if points_copy1[pnt_ind1]
          vert2=points_copy2[pnt_ind2] if points_copy2[pnt_ind2]
          new_vert=[0,0,0]
          if vert2
           if adj_curve.nil?
              new_vert.x=vert1.x+step*(vert2.x-vert1.x)/(steps)
              new_vert.y=vert1.y+step*(vert2.y-vert1.y)/(steps)
              new_vert.z=vert1.z+step*(vert2.z-vert1.z)/(steps)
            else
              x_pos=(adj_times*(step+0.1-0.1)/(steps+0.1-0.1))-((adj_times*(step+0.1-0.1)/(steps+0.1-0.1))).truncate
              crv_bnds=Geom::BoundingBox.new
              adj_curve.vertices.each{|vrt| crv_bnds.add vrt}
              if crv_bnds.depth==0
                x=crv_bnds.min.x+(x_pos*crv_bnds.width)
                for edg in adj_curve.edges
                  if (edg.start.position.x<=x and edg.end.position.x>=x) or (edg.start.position.x>=x and edg.end.position.x<=x) or ((edg.start.position.x<=x and edg.end.position.x<=x) and step==steps)
                    k=(edg.start.position.y-edg.end.position.y)/(edg.start.position.x-edg.end.position.x) if not edg.start.position.x-edg.end.position.x==0
                    b=edg.start.position.y-k*edg.start.position.x
                    if not edg.start.position.x-edg.end.position.x==0
                      y=k*x+b 
                    else
                      y=(edg.start.position.y+edg.end.position.y)/2
                    end
                    break
                  end
                end
                percnt=((y-crv_bnds.min.y)/crv_bnds.height).abs
              end
              new_vert.x=vert1.x+percnt*(vert2.x-vert1.x)
              new_vert.y=vert1.y+percnt*(vert2.y-vert1.y)
              new_vert.z=vert1.z+percnt*(vert2.z-vert1.z)
            end
          else
            new_vert=vert1
          end
          verts += [new_vert]
          pnt_ind +=1
        end
       
        if percnt
          clr_rat2=(254*percnt).truncate
          clr_rat1=(254-clr_rat2)
        else
          clr_rat2=254*step/(steps+1)
          clr_rat1=(255-clr_rat2)
        end
        
        #######################
        @rd=col2.red*clr_rat2/255+col1.red*clr_rat1/255 if col1 and col2
        @grn=col2.green*clr_rat2/255+col1.green*clr_rat1/255 if col1 and col2
        @bl=col2.blue*clr_rat2/255+col1.blue*clr_rat1/255 if col1 and col2
        @alp=mat2.alpha*clr_rat2/255+mat1.alpha*clr_rat1/255 if mat1 and mat2
        @blended_mat=[@rd, @grn, @bl, @alp]
        #######################
        @rd1=bk_col2.red*clr_rat2/255+bk_col1.red*clr_rat1/255 if bk_col1 and bk_col2
        @grn1=bk_col2.green*clr_rat2/255+bk_col1.green*clr_rat1/255 if bk_col1 and bk_col2
        @bl1=bk_col2.blue*clr_rat2/255+bk_col1.blue*clr_rat1/255 if bk_col1 and bk_col2
        @alp1=bk_mat2.alpha*clr_rat2/255+bk_mat1.alpha*clr_rat1/255 if bk_mat1 and bk_mat2
        @blended_back_mat=[@rd1, @grn1, @bl1, @alp1]
        #######################
        #exception handling added (ver 1.4) to pass the exception when not all vertices of an intermediate face are not at the same plane
        #when exception occures, script adds curves
        if not draw_surf_flag=="Surface only (suppress contours)"
          begin
            verts.uniq! #step 0 usually contains nonunic vertices
            if not step==steps+1
              new_face=contours_group.entities.add_face verts
              new_face.material=@blended_mat if @blended_mat[0] #condition added ver 1.8. to prevent freezing interactive objects with default materials faces
              new_face.back_material=@blended_back_mat if @blended_back_mat[0] #condition added ver 1.8. to prevent freezing interactive objects with default materials faces
              new_face.material.alpha=@blended_mat[3] if @blended_mat[0] #to enable alpha blending
              new_face.back_material.alpha=@blended_back_mat[3] if @blended_back_mat[0] #to enable alpha blending
            end 
          rescue
            begin
              cur_step_bb = Geom::BoundingBox.new
              verts.each{|vrt| cur_step_bb.add vrt}
              bound_cntr=cur_step_bb.center
              vrt_no=-1
              while vrt_no<verts.length
                new_face=contours_group.entities.add_face [verts[vrt_no],verts[vrt_no+1],bound_cntr]
                new_face.material=@blended_mat if @blended_mat[0] 
                new_face.back_material=@blended_back_mat if @blended_back_mat[0] 
                new_face.material.alpha=@blended_mat[3] if @blended_mat[0] 
                new_face.back_material.alpha=@blended_back_mat[3] if @blended_back_mat[0]
                vrt_no+=1
                new_face.edges.each{|edg| edg.soft=true; edg.smooth=true}
                @cont_grp_faces_cnt+=1 #counter added ver. 1.1
              end
            rescue
              @bad_int_face_cnt+=1 #counter added ver. 1.1
            end
            ensure
              new_curve=contours_group.entities.add_curve verts
              contours_group.entities.add_edges new_curve[0].start, new_curve[new_curve.length-1].end
              @curve_instead_cnt+=1 #counter added ver. 1.1
            end
        end
        step +=1
        @pb.update(step)

        if (draw_surf_flag=="Yes" or draw_surf_flag=="Surface only (suppress contours)") and verts1
          draw_surf verts,verts1,mesh_group,"Face"
        end
        
        if draw_tracks_flag=="Yes" and verts1
          draw_vert_tracks verts,verts1,tracks_group
        end
        verts1=verts
      end

      mesh=Geom::PolygonMesh.new
      
      #~ if not mesh_group.nil? and not mesh_group.deleted? and blend_skin_color=="Yes"
        #~ @pb1=Path_Face_ProgressBar.new(mesh_group.entities.length)
        #~ ent_ind=0
        #~ status=Sketchup.active_model.commit_operation
        #~ timer_id=UI.start_timer(0.001,true) {
          #~ face=mesh_group.entities[ent_ind]
          #~ status1=Sketchup.active_model.start_operation "Smoothing and Softing"
          #~ if face.respond_to? "edges"
            #~ face.edges.each{|edg|
              #~ edg.soft=true
              #~ edg.smooth=true
            #~ }
          #~ end
          #~ ent_ind+=1
          #~ @pb1.update(ent_ind)
          #~ Sketchup.active_model.active_view.invalidate
          #~ if ent_ind==mesh_group.entities.length
            #~ timer_id=UI.stop_timer timer_id 
            #~ status1=Sketchup.active_model.commit_operation
          #~ end
        #~ }
      #~ end

      #~ if blend_skin_color=="No" and not mesh_group.nil? and not mesh_group.deleted?
        #~ mesh_group.entities.each{|face|
          #~ pt1=face.vertices[0].position if face.vertices[0]
          #~ pt2=face.vertices[1].position if face.vertices[1]
          #~ pt3=face.vertices[2].position if face.vertices[2]
          #~ mesh.add_polygon pt1,pt2,pt3 if (pt1 and pt2 and pt3)
        #~ }
        #~ mesh_group.entities.clear!
        #~ mesh_group.entities.add_faces_from_mesh mesh, 12
      #~ end
      
      ###############################
      #    transforming back path and results of blending    #
      ###############################
      if align_flag=="Yes" or align_flag=="Yes and center to path" and path_is_flat and not path_is_line and not enforce_flag=="Enforce 3d algorithm"
        entities.transform_entities(path_transform, path)
        mesh_group.transform! path_transform if mesh_group and not mesh_group.deleted?
        tracks_group.transform! path_transform if tracks_group and not tracks_group.deleted?
        contours_group.transform! path_transform if contours_group and not contours_group.deleted?
      end
      
      ################################
      # Adding observer to handle changes of initial objects  #
      ################################
      if make_observer_flag==true
        ent1=@face1 if not @face1.nil?
        ent2=@face2 if not @face2.nil?   

        ent1.add_observer (PathFaceEntityObserver.new ent1,ent2,@path,draw_surf_flag, draw_tracks_flag, blend_skin_color,align_flag,enforce_flag,contours_group,mesh_group,tracks_group,adj_curve,adj_times) 
        ent2.add_observer (PathFaceEntityObserver.new ent1,ent2,@path,draw_surf_flag, draw_tracks_flag, blend_skin_color,align_flag,enforce_flag,contours_group,mesh_group,tracks_group,adj_curve,adj_times)
      end
      
      ##############################
      #   result statistics report
      ##############################
      @path_verts_cnt=path.vertices.length
      report_str=""
      report_str+="Path curve contains \t\t#{@path_verts_cnt} vertices \n" if @path_verts_cnt>0 and @path_verts_cnt
      report_str+="Faces/Contours group contains \t#{@cont_grp_faces_cnt} faces\n" if @cont_grp_faces_cnt>0
      report_str+="Mesh group contains \t\t#{@mesh_grp_faces_cnt} faces\n" if @mesh_grp_faces_cnt>0
      report_str+="Tracks group contains \t\t#{@tracks_grp_edges_cnt} edges\n" if @tracks_grp_edges_cnt>0
      report_str+="Curves instead of faces count:  \t#{@curve_instead_cnt} \n" if @curve_instead_cnt>0
      report_str+="'Bad' faces in contours group count: \t#{@bad_int_face_cnt} \n" if @bad_int_face_cnt>0
      report_str+="'Bad' faces in mesh skin group count: \t#{@bad_face_cnt} \n" if @bad_face_cnt>0
      report_str+="Alignment of initial faces is set to: \t'#{align_flag}' \n"
      report_str+="Skin mesh color blending is enabled \n" if blend_skin_color=="Yes"
      report_str+="Enforce flag is set to: \t\t#{enforce_flag} \n" if not enforce_flag=="..."
      puts report_str
      
      ##############################
      #   assigning attributes to groups
      ##############################     
      assgn_attrs_2_grp mesh_group,draw_surf_flag, draw_tracks_flag, blend_skin_color, align_flag,enforce_flag, face1.entityID, face2.entityID, @path.edges[0].entityID ,adj_curve,adj_times if @path
      assgn_attrs_2_grp tracks_group,draw_surf_flag, draw_tracks_flag, blend_skin_color, align_flag,enforce_flag, face1.entityID, face2.entityID, @path.edges[0].entityID ,adj_curve,adj_times if @path
      assgn_attrs_2_grp contours_group,draw_surf_flag, draw_tracks_flag, blend_skin_color, align_flag,enforce_flag, face1.entityID, face2.entityID, @path.edges[0].entityID ,adj_curve,adj_times if @path
      
      entities.transform_entities(@curve_transform, adj_curve) if not adj_curve.nil? and @curve_transform
      
      status = Sketchup.active_model.commit_operation
      
    end
    
    def assgn_attrs_2_grp (grp,draw_surf_flag, draw_tracks_flag, blend_skin_color, align_flag, enforce_flag, fc1, fc2, pth,adj_curve,adj_times)
      grp.set_attribute "pathface_attr_dict", "fc1", fc1
      grp.set_attribute "pathface_attr_dict", "fc2", fc2
      grp.set_attribute "pathface_attr_dict", "path", pth
      grp.set_attribute "pathface_attr_dict", "draw_surf_flag", draw_surf_flag
      grp.set_attribute "pathface_attr_dict", "draw_tracks_flag", draw_tracks_flag
      grp.set_attribute "pathface_attr_dict", "blend_skin_color", blend_skin_color
      grp.set_attribute "pathface_attr_dict", "align_flag", align_flag
      grp.set_attribute "pathface_attr_dict", "enforce_flag", enforce_flag
      grp.set_attribute "pathface_attr_dict", "adj_curve", adj_curve
      grp.set_attribute "pathface_attr_dict", "adj_times", adj_times
    end
    
    ###################
    #    surface drawing method      #
    ###################
    def draw_surf (points1, points2,group, face_flag)
      if points1.length>=points2.length
        max_pnts_cnt=points1.length
      else
        max_pnts_cnt=points2.length
      end
      pnt_ind=-1 if face_flag=="Face" #because the face is closed (first vetex coincides with last one)
      pnt_ind=0 if not face_flag=="Face"
      while pnt_ind<max_pnts_cnt
        pnt_ind1=(pnt_ind*points1.length/max_pnts_cnt).truncate
        pnt_ind2=(pnt_ind*points2.length/max_pnts_cnt).truncate
        vert1=points1[pnt_ind1]
        vert2=points2[pnt_ind2]
        pnt_ind3=((pnt_ind+1)*points1.length/max_pnts_cnt).truncate
        pnt_ind4=((pnt_ind+1)*points2.length/max_pnts_cnt).truncate
        vert3=points1[pnt_ind3]
        vert4=points2[pnt_ind4]
        #exception handling added for complex curves processing (complex geometry generates some faces with duplicating vertices and Skethup can not add them)
        begin
          if (not vert1.nil? and not vert2.nil? and not vert3.nil? and not vert1==vert3)
            mesh_face1=group.entities.add_face vert1,vert2,vert3
            mesh_face1.edges.each{|edg| 
              edg.soft=true
              edg.smooth=true
            }
            if not @blended_mat.nil?
              mesh_face1.material=@blended_mat if not @blended_mat[0].nil?
              mesh_face1.back_material=@blended_back_mat if not @blended_back_mat[0].nil?
              
              mesh_face1.material.alpha=@blended_mat[3] if not @blended_mat[0].nil? and mesh_face1.material
              mesh_face1.back_material.alpha=@blended_back_mat[3] if not @blended_back_mat[0].nil? and mesh_face1.back_material
            end
          end
        rescue
          #puts "Some 'bad' faces were ignored."
          @bad_face_cnt+=1
        end
        begin
          if (not vert2.nil? and not vert4.nil? and not vert3.nil? and not vert2==vert4)
            mesh_face2=group.entities.add_face vert3,vert4,vert2
            mesh_face2.edges.each{|edg| 
              edg.soft=true
              edg.smooth=true
            }
            if not @blended_back_mat.nil?  
              mesh_face2.material=@blended_mat if not @blended_mat[0].nil?
              mesh_face2.back_material=@blended_back_mat if not @blended_back_mat[0].nil?
              
              mesh_face2.material.alpha=@blended_mat[3] if not @blended_mat[0].nil? and mesh_face2.material
              mesh_face2.back_material.alpha=@blended_back_mat[3] if not @blended_back_mat[0].nil? and mesh_face2.back_material
            end
          end
          @mesh_grp_faces_cnt+=2
        rescue
          #puts "Some 'bad' faces were ignored."
          @bad_face_cnt+=1
        end
        pnt_ind +=1
      end
    end
    #######################
    #    vertices tracks drawing method     #
    #######################
    def draw_vert_tracks (pnts1, pnts2,group) 
      points1=pnts1.uniq
      points2=pnts2.uniq
      if points1.length>=points2.length
        max_pnts_cnt=points1.length
      else
        max_pnts_cnt=points2.length
      end
      pnt_ind=0
      while pnt_ind<max_pnts_cnt
        pnt_ind1=(pnt_ind*points1.length/max_pnts_cnt).truncate
        pnt_ind2=(pnt_ind*points2.length/max_pnts_cnt).truncate
        vert1=points1[pnt_ind1]
        vert2=points2[pnt_ind2]
        #~ begin
          group.entities.add_edges vert1,vert2
        #~ rescue
          #~ puts "Some 'bad' edges were ignored."
        #~ end
        pnt_ind +=1
        @tracks_grp_edges_cnt+=1
      end
    end
  end

class Lss_PathFace_Sel_Tool
  def initialize
    @selection=Sketchup.active_model.selection
    @b_p=nil
    @blend_grp=@selection[0]
    @selection.clear
  end
  
  def activate
    
  end
  
  def onLButtonUp(flags, x, y, view)
    ph=view.pick_helper
    ph.do_pick x,y
    @b_p = ph.best_picked
    if not @b_p.nil?
      puts "#{@b_p.typename} is picked."
    else
      puts "Nothing is picked"
    end
    Sketchup.active_model.select_tool nil if @b_p.respond_to? "curve"
  end
  
  def deactivate(view)
    entities=Sketchup.active_model.entities
    @selection.add @b_p.all_connected
    adj_curve=@b_p.curve
    
    fc1_id=@blend_grp.get_attribute "pathface_attr_dict", "fc1"
    fc2_id=@blend_grp.get_attribute "pathface_attr_dict", "fc2"
    for ent in entities
      if ent.respond_to? "entityID"
        if fc1_id.to_s==ent.entityID.to_s
          fc1=ent
          break
        end
      end
    end
    for ent in entities
      if ent.respond_to? "entityID"
        if fc2_id.to_s==ent.entityID.to_s
          fc2=ent
          break
        end
      end
    end
    pth_edg_id=@blend_grp.get_attribute "pathface_attr_dict", "path"
    for ent in entities
      if ent.respond_to? "entityID"
        if pth_edg_id.to_s==ent.entityID.to_s
          pth=ent.curve
          break
        end
      end
    end
    align_flag=@blend_grp.get_attribute "pathface_attr_dict", "align_flag"
    
    mesh_group=nil
    tracks_group=nil
    contours_group=nil
    
    mesh_group=@blend_grp if @blend_grp.description.to_s=="lss pathface mesh"
    tracks_group=@blend_grp if @blend_grp.description.to_s=="lss pathface tracks"
    contours_group=@blend_grp if @blend_grp.description.to_s=="lss pathface contours"    
    
    if mesh_group
      draw_surf_flag="Surface only (suppress contours)" 
    else
      draw_surf_flag="No"
    end
    if tracks_group
      draw_tracks_flag="Yes"
    else
      draw_tracks_flag="No"
    end
    blend_skin_color=@blend_grp.get_attribute "pathface_attr_dict", "blend_skin_color"
    adj_times=@blend_grp.get_attribute "pathface_attr_dict", "adj_times"
    enforce_flag=@blend_grp.get_attribute "pathface_attr_dict", "enforce_flag"
    prompts=["Adjustment curve repeating times: "]
    defaults=[adj_times]
    adj_times=UI.inputbox prompts, defaults, "Repeating Times Setting"
    @blend_grp.entities.clear!
    lss_path_face=Lss_Path_Face.new
    lss_path_face.generate draw_surf_flag, draw_tracks_flag, blend_skin_color, align_flag, enforce_flag, fc1, fc2, pth, contours_group,mesh_group,tracks_group,adj_curve,adj_times[0]
  end
end

selection=Sketchup.active_model.selection
if( not file_loaded?("pathface.rb") )
    UI.add_context_menu_handler do |context_menu| 
      if selection.count>1
       context_menu.add_separator 
       context_menu.add_item("2 faces + path...") {
          if selection.count<3
        UI.messagebox "It is necessary to select two faces and one path curve."
        return
      end

      prompts = ["Connect steps with surface ", "Draw vertices tracks ", "Process 'skin' color ", "Align faces to path ", "3D path options "]
      defaults= ["No", "No", "No", "Yes", "..."]
      list = ["No|Yes|Surface only (suppress contours)", "No|Yes", "No|Yes", "No|Yes|Yes and center to path|No but center to path", "...|Enforce 3d algorithm|Ignore flat parts orientation"]
      settings=UI.inputbox prompts, defaults, list, "Settings"
      lss_path_face=Lss_Path_Face.new

      lss_path_face.selection_filter
      lss_path_face.generate settings[0], settings[1], settings[2], settings[3], settings[4]
        }
      end
   end
    
    UI.add_context_menu_handler do |context_menu| 
      if selection.count==1 and selection[0].respond_to? "description"
        if selection[0].description[0,12]=="lss pathface"
         context_menu.add_separator 
         context_menu.add_item("Pick pathface adjustment curve") {
            Sketchup.active_model.select_tool Lss_PathFace_Sel_Tool.new
          }
        end
      end
    end
    
   tool_menu = UI.menu "Plugins" 
   tool_menu.add_item("2 faces + path...") {
      if selection.count<3
        UI.messagebox "It is necessary to select two faces and one path curve."
        return
      end

      prompts = ["Connect steps with surface ", "Draw vertices tracks ", "Process 'skin' color ", "Align faces to path ", "3D path options "]
      defaults= ["No", "No", "No", "Yes", "..."]
      list = ["No|Yes|Surface only (suppress contours)", "No|Yes", "No|Yes", "No|Yes|Yes and center to path|No but center to path", "...|Enforce 3d algorithm|Ignore flat parts orientation"]
      settings=UI.inputbox prompts, defaults, list, "Pathface Settings"
      lss_path_face=Lss_Path_Face.new

      lss_path_face.selection_filter
      lss_path_face.generate settings[0], settings[1], settings[2], settings[3], settings[4]
    }
  end
  file_loaded("pathface.rb")